<?php

namespace Drupal\gutenberg\Theme;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Theme\ThemeNegotiatorInterface;
use Symfony\Component\Routing\Route;

/**
 * Defines a theme negotiator to force the default theme.
 *
 * Specifically this is used to override the admin theme being activated on
 * AJAX requests made with Drupal.ajax(). Drupal ajax requests send along the
 * currently active theme (i.e. the admin theme when editing in gutenberg) which
 * is then used to render the response. This negotiator ensures that the default
 * theme is used instead which we want when we're rendering blocks in gutenberg.
 */
class GutenbergThemeNegotiator implements ThemeNegotiatorInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new GutenbergThemeNegotiator.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public function applies(RouteMatchInterface $route_match) {
    $route = $route_match->getRouteObject();
    if (!$route instanceof Route) {
      return FALSE;
    }

    // Any route with this option set will force the default theme.
    $option = $route->getOption('_gutenberg_use_default_theme');
    if (!$option) {
      return FALSE;
    }

    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function determineActiveTheme(RouteMatchInterface $route_match) {
    // Just return the default theme.
    return $this->configFactory->get('system.theme')->get('default');
  }

}
